#!/usr/bin/env python3

import heapq
import os
import re
import sys

def merge_logs(*logs):
    files = [open(file, 'r') for file in logs]
    heap = []
    count = 0

    pattern = re.compile(r"^\[\s*(\d+\.\d+)\]")

    for file in files:
        entry = ()
        while True:
            line = file.readline()
            if not line:
                # Add the last one if multiline
                if entry:
                    heapq.heappush(heap, entry)
                file.close()
                break;
            line = line.rstrip('\n')
            match = pattern.match(line)
            if match:
                if entry:
                    heapq.heappush(heap, entry)
                # Make it sorted by timestamp but also insertion order
                # so that's why we use count here. See more from heapq
                # documentation (specifically the Priority Queue section).
                entry = (float(match.group(1)), count, line)
                count += 1
            else:
                entry = (entry[0], entry[1], entry[2] + "\n" + line)

    while heap:
        print(heapq.heappop(heap)[2])

if __name__ == "__main__":
    logs = sys.argv[1:]
    if not logs:
        print("Usage: " + os.path.basename(__file__) + " LOG1 [LOG2...]")
        print()
        print("Merges two or more log files into one")
        sys.exit(1)

    merge_logs(*logs)



import datetime

# Convert hex string into integer
def convert_int(fields, name):
    tmp = fields[name]
    if tmp:
        fields[name] = int(tmp, 16)

# Convert timestamp into datetime
def convert_timestamp(fields, name):
    tmp = fields[name]
    if tmp:
        dt = datetime.datetime.fromtimestamp(float(tmp))
        fields[name] = dt.strftime("%Y-%m-%dT%H:%M:%S")



#!/usr/bin/env python3

import argparse
import csv
import os
import util

def process_devices(args):
    base = os.path.basename(args.devices)
    base = os.path.splitext(base)[0]
    devices = base + '.processed.csv'

    with open(args.devices) as input_file, open(devices, 'w') as output_file:
        reader = csv.DictReader(input_file)
        field_names = [
            'domain',
            'route',
            'adapter',
            'index',
            'vendor',
            'device',
            'vendor_name',
            'device_name',
            'type',
            'generation',
        ]
        devices_writer = csv.DictWriter(output_file, fieldnames=field_names)
        devices_writer.writeheader()
        for row in reader:
            if row['type'] == 'Domain':
                continue
            util.convert_int(row, 'route')
            util.convert_int(row, 'vendor')
            util.convert_int(row, 'device')
            devices_writer.writerow(row)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog='process-devices',
                                     description='Process tblist -S output suitable for database.',
                                     epilog="""
Processes the tblist -S output into a format that is suitable for
inserting into a database. From file DEVICES generates a file called
DEVICES.processed.csv.
                                    """)
    parser.add_argument('devices');
    args = parser.parse_args()

    process_devices(args)



#!/usr/bin/env python3

import argparse
import csv
import os
import sys
import util

def write_row(fields, writer, field_names):
    for field in fields:
        f = {key: field[key] for key in field_names if key in field}
        # Convert to integer
        util.convert_int(f, 'offset')
        util.convert_int(f, 'data_offset')
        util.convert_int(f, 'value')
        # Write them
        writer.writerow(f)

def process_trace(args):
    base = os.path.basename(args.trace)
    base = os.path.splitext(base)[0]
    entries = base + '.processed.entries.csv'
    fields = base + '.processed.fields.csv'

    with open(args.trace) as input_file, \
         open(entries, 'w') as entries_file, \
         open(fields, 'w') as fields_file:
        reader = csv.DictReader(input_file)
        entry_names = [
            'entry',
            'timestamp',
            'datetime',
            'function',
            'dropped',
            'pdf',
            'cs',
            'domain',
            'route',
            'adapter',
            'adapter_type',
        ]
        entries_writer = csv.DictWriter(entries_file, fieldnames=entry_names)
        entries_writer.writeheader()
        field_names = [
            'entry',
            'offset',
            'data_offset',
            'value',
            'name',
        ]
        fields_writer = csv.DictWriter(fields_file, fieldnames=field_names)
        fields_writer.writeheader()
        line = None
        for row in reader:
            if not line:
                # First line in the file
                line = row['entry']
                fields = [row]
            else:
                if line != row['entry']:
                    # Write the entries first
                    e = {key: row[key] for key in entry_names if key in row}
                    util.convert_timestamp(e, 'datetime')
                    util.convert_int(e, 'route')
                    entries_writer.writerow(e)
                    # Then the fields
                    write_row(fields, fields_writer, field_names)
                    line = row['entry']
                    fields = [row]
                else:
                    fields.append(row)

        # Last one
        write_row(fields, fields_writer, field_names)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog='process-trace',
                                     description='Split the trace CSV output into separate files.',
                                     epilog="""
Splits trace output into separate files, more suitable for inserting
into database. From file TRACE generates two files
TRACE.processed.entries.csv and TRACE.processed.fields.csv. These have
matching first field that is the trace line number.
                                     """)
    parser.add_argument('trace');
    args = parser.parse_args()

    process_trace(args)
